import pytest
from instiz.helper import return_int_from_image_url


def test_return_int_from_image_url():
    assert (
        return_int_from_image_url(
            "http://cfs.tistory.com/custom/blog/44/444693/skin/images/1.png"
        )
        == 1
    )




HEADERS = {
    "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:63.0) Gecko/20100101 Firefox/63.0",
    "Referer": "https://ichart.instiz.net",
}

URL = "http://www.instiz.net/iframe_ichart_score.htm"

CHANGES = {
    "arrow1": "up",
    "arrow2": "down",
    "arrow3": "none",
    "arrow4": "new",
    "arrow5": "new",
}




import attr


@attr.s
class Entry(object):
    title = attr.ib(type=str)
    artist = attr.ib(type=str)
    rank = attr.ib(type=int)
    change_class = attr.ib(type=str)
    change_amount = attr.ib(type=int)
    score = attr.ib(type=int)

    @property
    def change(self):
        return {"class": self.change_class, "amount": self.change_amount}

    def nice_title(self):
        return self.artist + " - " + self.title



from bs4 import BeautifulSoup
from re import compile
from requests import get
from .CONSTANTS import CHANGES, HEADERS, URL
from .helper import return_int_from_image_url as int_parse
from .models import Entry


class iChart:
    def __init__(self):
        self.entries = []
        self.doc = None
        self._generator = None
        self._post_init()

    def get_next_entry(self):
        """Function that returns the next entry of the iChart onwards"""
        self._add_to_list()
        return self.entries[-1]

    def realtime_top_10(self):
        """Function that returns a list of current top 10 entries on iChart"""
        while len(self.entries) < 10:
            self._add_to_list()
        return self.entries[:10]

    def refresh(self):
        """
        Refreshes the instance.
        Returns self, so the call can be inlined with other function call
        """
        self._make_request()
        self.entries = []
        return self

    def _add_to_list(self):
        self.entries.append(next(self._generator))

    def _make_request(self):
        request = get(URL, headers=HEADERS, timeout=15)
        self.doc = BeautifulSoup(request.text, "lxml")

    def _entry_iterator(self):
        result_set = self.doc.find_all(
            class_=compile("spage_score_item(_1st)*")
        )
        for div in result_set:
            yield self._parse_score(div)

    def _parse_score(self, div):
        title = div.find(class_=compile("ichart_score[0-9]*_song1")).text
        artist = div.find(class_=compile("ichart_score[0-9]*_artist1")).text
        rank = int_parse(
            div.find(class_=compile("ichart_score[0-9]*_rank")).img["src"]
        )
        change = div.find(class_=compile("ichart_score[0-9]*_change"))
        change_class = CHANGES[change.span["class"][1]]
        change_rank = int(change.text) if change.text != "" else 0
        score_div = div.find(
            class_=compile("ichart_score[0-9]*_score")
        ).find_all("img")
        score = int(
            "".join([str(int_parse(score["src"])) for score in score_div])
        )
        return Entry(title, artist, rank, change_class, change_rank, score)

    def _post_init(self):
        self._make_request()
        self._generator = self._entry_iterator()




import pytest
from instiz import iChart


@pytest.fixture
def class_init():
    ichart = iChart()
    return ichart


def test_10(class_init):
    assert len(class_init.realtime_top_10()) == 10


def test_refresh(class_init):
    x = class_init.refresh()
    assert isinstance(x, iChart)


def test_refresh_clear_entries(class_init):
    class_init.realtime_top_10()
    class_init.refresh()
    assert len(class_init.entries) == 0


def test_get_rank(class_init):
    x = class_init.get_next_entry()
    assert x.rank == 1


def test_changes_in_entry(class_init):
    x = class_init.get_next_entry()
    assert "class" in x.change


def test_nice_title_formatting(class_init):
    x = class_init.get_next_entry()
    assert x.nice_title() == x.artist + " - " + x.title



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="instiz",
    version="0.1.0",
    author="Jonathan Nicholas",
    author_email="Jonathan.Nicholas@protonmail.com",
    description="A Python3-only library for iChart K-Pop chart scores.",
    install_requires=[
        "requests>=2.20.0",
        "beautifulsoup4>=4.6.3",
        "attrs>=18.2.0",
        "lxml>=4.2.5",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Chromadream/instiz",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "Topic :: Software Development :: Libraries",
    ],
)
